source('/esarchive/scratch/vtorralba/vtesis_scripts/weather_regimes/CompositeVero.R')

#' 
#'@param data an array containing anomalies with named dimensions with at least start date 'sdate', forecast time 'ftime', latitude 'lat' and longitude 'lon'.
#'   
RegimesAssign <- function(data, ref_maps, lats, method = 'distance', composite=FALSE) {
  
  if (is.null(names(dim(data)))) {
    stop("Parameter 'data' must be an array with named dimensions.")
  }
  
  if (is.null(names(dim(ref_maps)))) {
    stop("Parameter 'ref_maps' must be an array with named dimensions.")
  }
  
  if (is.null(lats)) {
    stop("Parameter 'lat' must be specified.")
  }
  
  if (length(lats) != dim(data)['lat'] | (length(lats) != dim(ref_maps)['lat']) ) {
    stop(" Parameter 'lats' does not match with the dimension 'lat' in the 
         parameter 'data' or in the parameter 'ref_maps'.")
  }
  
  dimData <- names(dim(data))
  
  if ('sdate' %in% dimData && 'ftime' %in% dimData) {
    nsdates <- dim(data)['sdate']
    nftimes <- dim(data)['ftime']
    data <- MergeDims(data, 
                      merge_dims = c('ftime','sdate'), 
                      rename_dim = 'time')
  } else if ('sdate' %in% dimData | 'ftime' %in% dimData) {
    names(dim(data))[which(dimData == 'sdate' | dimData == 'ftime') ] = 'time' 
  } else {
    if (!('time' %in% dimData)) {
      stop("Parameter 'data' must have temporal dimensions.")
    }
  }
  
  index <- Apply( data = list(target = data),
                  target_dims = c('lat','lon'),
                  fun = '.RegimesAssign',
                  ref = ref_maps,
                  lat = lats,  method = method)[[1]]
  
  
  nclust <- dim(ref_maps)['nclust']
  freqs <- rep(NA, nclust)
  for (n in 1:nclust) {
    freqs[n] <- (length(which(index == n)) / length(index)) * 100
  }
  
  if (composite){
    poslon <- which(names(dim(data))=='lon')
    poslat <- which(names(dim(data))=='lat')
    postime <- which(names(dim(data))=='time')
    
    
    if (any(is.na(index))) {
      recon <-
        list(
          composite = array(NA,
            dim = c(dim(ref_maps)['lon'], dim(ref_maps)['lat'], dim(ref_maps)['nclust'])),
          pvalue = array(NA, 
            dim = c(dim(ref_maps)['lon'], dim(ref_maps)['lat'], dim(ref_maps)['nclust'])))
    } else {
      recon <- Composite(var = aperm(data, c(poslon,poslat,postime)), occ = index, K=dim(ref_maps)['nclust'])
    }

    output <- list(composite = recon$composite,
                   pvalue = recon$pvalue,
                   cluster = index,
                   frequency = freqs)
  } else{
    
    output <- list(cluster = index,
                   frequency = freqs)
  }
  
  return(output)
}

.RegimesAssign <- function(ref, target, method = 'distance', lat, composite=FALSE) {
  posdim <- which(names(dim(ref)) == 'nclust')
  poslat <- which(names(dim(ref)) == 'lat')
  poslon <- which(names(dim(ref)) == 'lon')
  
  nclust <- dim(ref)[posdim]
  
  if (all(dim(ref)[-posdim] != dim(target))) {
    stop('The target should have the same dimensions [lat,lon] that
         the reference ')
  }
  
  if (is.null(names(dim(ref))) | is.null(names(dim(target)))) {
    stop(
      'The arrays should include dimensions names ref[nclust,lat,lon]
      and target [lat,lon]'
    )
  }

  
  if (length(lat) != dim(ref)[poslat]) {
    stop('latitudes do not match with the maps')
  }

 if (is.na(max(target))){
    assign <- NA

  }else{

  
  # This dimensions are reorganized
  ref <- aperm(ref, c(posdim, poslat, poslon))
  target <-
    aperm(target, c(which(names(dim(
      target
    )) == 'lat'), which(names(dim(
      target
    )) == 'lon')))
  
  # weights are defined
  latWeights <- InsertDim(sqrt(cos(lat * pi / 180)), 2, dim(ref)[3])
  
  
  rmsdiff <- function(x, y) {
    dims <- dim(x)
    ndims <- length(dims)
    if (ndims != 2 | ndims != length(dim(y))) {
      stop('x and y should be maps')
    }
    map_diff <- NA * x
    for (i in 1:dims[1]) {
      for (j in 1:dims[2]) {
        map_diff[i, j] <- (x[i, j] - y[i, j]) ^ 2
      }
    }
    rmsdiff <- sqrt(mean(map_diff))
    return(rmsdiff)
  }
  
  if (method == 'ACC') {
    corr <- rep(NA, nclust)
    for (i in 1:nclust) {
      corr[i] <-
        ACC(InsertDim(InsertDim(
          InsertDim(ref[i, , ] * latWeights, 1, 1), 2, 1
        ), 3, 1),
        InsertDim(InsertDim(
          InsertDim(target * latWeights, 1, 1), 2, 1
        ), 3, 1))$ACC[2]
    }
    assign <- which(corr == max(corr))
  }
  
  if (method == 'distance') {
    rms <- rep(NA, nclust)
    for (i in 1:nclust) {
      rms[i] <- rmsdiff(ref[i, , ] * latWeights, target * latWeights)
    }
    assign <- which(rms == min(rms))
  }
  }
  
  return(assign)
}

